<?php
/**
 * Class for getting and setting db/default values
 * 
 * @category Admin
 * @author   Yatmo SRL
 */

// Exit if accessed directly
if (!defined('ABSPATH')) {
    exit;
}

require_once yatmo_text__PLUGIN_DIR . 'class.plugin-option.php';

// TODO: add option to reset just a single field

/**
 * Used to get and set values
 * 
 * Features:
 * * Add prefixes to db options
 * * built-in admin settings page method
 */
class yatmo_text_Plugin_Settings
{
    /**
     * Prefix for options, for unique db entries
     * 
     * @var string $prefix
     */
    public $prefix = 'yatmo_';
    
    /**
     * Singleton instance
     * 
     * @var yatmo_text_Plugin_Settings
     **/
    private static $_instance = null;

    /**
     * Default values and admin form information
     * Needs to be created within __construct
     * in order to use a function such as __()
     * 
     * @var array $options
     */
    public $options = array();

    /**
     * Singleton
     * 
     * @static
     * 
     * @return yatmo_text_Plugin_Settings
     */
    public static function init() {
        if ( !self::$_instance ) {
            self::$_instance = new self;
        }

        return self::$_instance;
    }

    /**
     * Instantiate the class
     */
    private function __construct() 
    {

        /* update yatmo version from main class */
        $yatmo_version = yatmo_text::$yatmo_version;

        $foreachtext = __('You can also change this for each text', 'yatmo-text');

        /* 
        * initiate options using internationalization! 
        */
        $this->options = array(
            'default_language' => array(
                'display_name'=>__('Default language', 'yatmo-text'),
                'default'=>'EN',
                'type' => 'select',
                'options' => array(
                    'EN' => 'EN (English)',
                    'FR' => 'FR (Français)',
                    'IT' => 'IT (Italiano)',
                    'ZH' => 'ZH (汉语)',
                    'DE' => 'DE (Deutsch)',
                    'ES' => 'ES (Español)',
                    'HI' => 'HI (हिन्दी)',
                    'JA' => 'JA (日本語)',
                    'PT' => 'PT (Português)',
                    'NL' => 'NL (Nederlands)',
                    'CA' => 'CA (Català)',
                    'RU' => 'RU (Русский)',
                ),
                'helptext' => sprintf(
                    '%1$s %2$s <br /> <code>[yatmo-text language="EN"]</code>', 
                    __('Default language for generated texts.', 'yatmo-text'),
                    $foreachtext
                )
            ),
            'default_openingtitlenodetype' => array(
                'display_name'=>__('Default opening HTML title node', 'yatmo-text'),
                'default'=>'<h3>',
                'type' => 'text',
                'helptext' => sprintf(
                    '%1$s %2$s <br /> <code>[yatmo-text openingtitlenodetype="&lt;h4 style=&quot;color=green;&quot;&gt;"]</code>', 
                    __('Default opening HTML node for paragraphs titles (be careful to keep it valid if you change it!).', 'yatmo-text'),
                    $foreachtext
                )
            ),
            'default_closingtitlenodetype' => array(
                'display_name'=>__('Default closing HTML title node', 'yatmo-text'),
                'default'=>'</h3>',
                'type' => 'text',
                'helptext' => sprintf(
                    '%1$s %2$s <br /> <code>[yatmo-text closingtitlenodetype="&lt;/h4&gt;"]</code>', 
                    __('Default closing HTML node for paragraphs titles (be careful to keep it valid if you change it!).', 'yatmo-text'),
                    $foreachtext
                )
            ),
            'default_openingparagraphnodetype' => array(
                'display_name'=>__('Default opening HTML paragraph node', 'yatmo-text'),
                'default'=>'<p>',
                'type' => 'text',
                'helptext' => sprintf(
                    '%1$s %2$s <br /> <code>[yatmo-text openingparagraphnodetype="&lt;p&gt;"]</code>', 
                    __('Default opening HTML node for paragraphs (be careful to keep it valid if you change it!).', 'yatmo-text'),
                    $foreachtext
                )
            ),
            'default_closingparagraphnodetype' => array(
                'display_name'=>__('Default closing HTML paragraph node', 'yatmo-text'),
                'default'=>'</p>',
                'type' => 'text',
                'helptext' => sprintf(
                    '%1$s %2$s <br /> <code>[yatmo-text closingparagraphnodetype="&lt;/p&gt;"]</code>', 
                    __('Default closing HTML node for paragraphs (be careful to keep it valid if you change it!).', 'yatmo-text'),
                    $foreachtext
                )
            ),
            'default_openingkeywordnodetype' => array(
                'display_name'=>__('Default opening HTML keyword/strong node', 'yatmo-text'),
                'default'=>'<strong class="yatmoKeyword">',
                'type' => 'text',
                'helptext' => sprintf(
                    '%1$s %2$s <br /> <code>[yatmo-text openingkeywordnodetype="&lt;strong class=&quot;keyword&quot;&gt;"]</code>', 
                    __('Default opening HTML node for keywords (be careful to keep it valid if you change it! Put an empty string if you don\'t want to have a different look for keywords).', 'yatmo-text'),
                    $foreachtext
                )
            ),
            'default_closingkeywordnodetype' => array(
                'display_name'=>__('Default closing HTML keyword/strong node', 'yatmo-text'),
                'default'=>'</strong>',
                'type' => 'text',
                'helptext' => sprintf(
                    '%1$s %2$s <br /> <code>[yatmo-text closingkeywordnodetype="&lt;/strong&gt;"]</code>', 
                    __('Default closing HTML node for keywords (be careful to keep it valid if you change it!).', 'yatmo-text'),
                    $foreachtext
                )
            ),
            'default_visibleparagraphs' => array(
                'display_name'=>__('Default visible paragraphs', 'yatmo-text'),
                'default'=>'education,shopping,publictransports,transports,cities,dailylife,medical,entertainment,tourism',
                'type' => 'text',
                'helptext' => sprintf(
                    '%1$s %2$s <br /> <code>[yatmo-text visibleparagraphs="education,publictransports,transports"]</code>', 
                    __('Default paragraphs you want to show (complete list: education,shopping,publictransports,transports,cities,dailylife,medical,entertainment,tourism). FYI: the paragraphs of our generated texts are randomly ordered!', 'yatmo-text'),
                    $foreachtext
                )
            ),
			'default_showstreetnameintitle' => array(
                'display_name'=>__('Show street name in first title', 'yatmo-text'),
                'default'=>'1',
                'type' => 'checkbox',
                'helptext' => sprintf(
                    '%1$s %2$s <br /> <code>[yatmo-text showstreetnameintitle="1"]</code>', 
                    __('Default configuration to show the detected street name in the first title (our API tries to detect the correct street: it won\'t be in the title if not found). If you want to keep the precise location anonymous, it\'s a good idea to uncheck this option :-)', 'yatmo-text'),
                    $foreachmap
                )
            ),
			'default_showplacenameintitle' => array(
                'display_name'=>__('Show place name in second title', 'yatmo-text'),
                'default'=>'1',
                'type' => 'checkbox',
                'helptext' => sprintf(
                    '%1$s %2$s <br /> <code>[yatmo-text showplacenameintitle="1"]</code>', 
                    __('Default configuration to show the detected place name in the second title (our API tries to detect the correct place: it won\'t be in the title if not found).', 'yatmo-text'),
                    $foreachmap
                )
            ),
            'yatmo_key' => array(
                'display_name'=>__('Yatmo key (optional!)', 'yatmo-text'),
                'default' => '',
                'type' => 'text',
                'noreset' => true,
                'helptext' => sprintf(
                    '%1$s <a href="https://yatmo.com/en/account/register" target="_blank"> %2$s</a>, %3$s',
                    __('You don\'t need a key to use our text generator API except if you exceed <strong>15 new generated texts per day</strong> or if you want to hide our logo at the bottom of the text. If you need a key, you can create an account ', 'yatmo-text'),
                    __('here', 'yatmo-text'),
                    __('then supply the key here.', 'yatmo-text')
                )
            ),
        );

        foreach ($this->options as $name => $details) {
            $this->options[ $name ] = new yatmo_text_Plugin_Option($details);
        }
    }

    /**
     * Wrapper for WordPress get_options (adds prefix to default options)
     *
     * @param string $key                
     * 
     * @return varies
     */
    public function get($key) 
    {
        $default = $this->options[ $key ]->default;
        $key = $this->prefix . $key;
        return get_option($key, $default);
    }

    /**
     * Wrapper for WordPress update_option (adds prefix to default options)
     *
     * @param string $key   Unique db key
     * @param varies $value Value to insert
     * 
     * @return yatmo_text_Plugin_Settings
     */
    public function set ($key, $value) {
        $key = $this->prefix . $key;
        update_option($key, $value);
        return $this;
    }

    /**
     * Wrapper for WordPress delete_option (adds prefix to default options)
     *
     * @param string $key Unique db key
     * 
     * @return boolean
     */
    public function delete($key) 
    {
        $key = $this->prefix . $key;
        return delete_option($key);
    }

    /**
     * Delete all options
     *
     * @return yatmo_text_Plugin_Settings
     */
    public function reset()
    {
        foreach ($this->options as $name => $option) {
            if (
                !property_exists($option, 'noreset') ||
                $option->noreset != true
            ) {
                $this->delete($name);
            }
        }
        return $this;
    }
}
